<?php
/**
 * Copyright © 2018 Filoblu S.p.A. All rights reserved.
 */

namespace FiloBlu\MappConnector\Model\From\Entity;

use Exception;
use FiloBlu\MappConnector\Helper\FileLoggerHelper;
use FiloBlu\MappConnector\Model\Config;
use FiloBlu\MappConnector\Model\To\Mapp\SFTP;
use Magento\Catalog\Api\ProductRepositoryInterface;
use Magento\Eav\Api\AttributeSetRepositoryInterface;
use Magento\Framework\App\ResourceConnection;
use Magento\Framework\Exception\NoSuchEntityException;
use Magento\Framework\UrlInterface;
use Magento\Sales\Api\Data\OrderInterface;
use Magento\Sales\Api\OrderRepositoryInterface;
use Magento\Sales\Model\Order\Item;
use Magento\Store\Model\StoreManagerInterface;

/**
 * Class OrderItem
 * @package FiloBlu\MappConnector\Model\From\Entity
 */
class OrderItem extends AbstractEntity implements EntityInterface
{

    /**
     * Entity
     *
     * @var string
     */
    protected $_entity = 'order_item';

    /**
     * Magento Entity Model
     *
     * @var string
     */
    protected $_entityModel = Item::class;

    /**
     * Magento Entity Model Id
     *
     * @var string
     */
    protected $_entityModelPrimary = 'order_id';

    /**
     * Magento Entity Model Related Id
     *
     * @var string
     */
    protected $_entityModelRelatedPrimary = 'order_id';

    /**
     * Mapp Resource Model
     *
     * @var string
     */
    protected $_entityMappResource = SFTP::class;

    /**
     * Mapp Resource Model Id
     *
     * @var array
     */
    protected $_entityMappPrimary = ['primaryDB' => 'order_id', 'primaryMapp' => 'email'];

    /**
     * @var string
     */
    protected $_xmlConfigPath = 'mappconnector/order_mapping/order_item';

    /**
     * @var
     */
    protected $_order;

    /**
     * @var FileLoggerHelper
     */
    protected $_fileLogger;

    /**
     * @var StoreManagerInterface
     */
    protected $_storeManager;

    /**
     * @var AttributeSetRepositoryInterface
     */
    protected $_attributeSet;

    /**
     * @var ProductRepositoryInterface
     */
    protected $_productRepository;
    /**
     * @var Config
     */
    protected $_config;
    /**
     * @var OrderRepositoryInterface
     */
    private $orderRepository;

    /**
     * OrderItem constructor.
     * @param Order $order
     * @param Config $config
     * @param FileLoggerHelper $fileLoggerHelper
     * @param StoreManagerInterface $storeManager
     * @param AttributeSetRepositoryInterface $attributeSet
     * @param ProductRepositoryInterface $productRepository
     * @param OrderRepositoryInterface $orderRepository
     * @param ResourceConnection $resourceConnection
     */
    public function __construct(
        Order $order,
        Config $config,
        FileLoggerHelper $fileLoggerHelper,
        StoreManagerInterface $storeManager,
        AttributeSetRepositoryInterface $attributeSet,
        ProductRepositoryInterface $productRepository,
        OrderRepositoryInterface $orderRepository,
        ResourceConnection $resourceConnection
    ) {
        parent::__construct($resourceConnection);
        $this->_config = $config;
        $this->_order = $order;
        $this->_fileLogger = $fileLoggerHelper;
        $this->_storeManager = $storeManager;
        $this->_attributeSet = $attributeSet;
        $this->_productRepository = $productRepository;
        $this->orderRepository = $orderRepository;
    }

    /**
     * {@inheritDoc}
     */
    public function getDefaultData($record)
    {
        $result = [];

        //Get Store
        $store = $this->_storeManager->getStore($record['store_id']);

        //Get Order
        $order = $this->orderRepository->get($record['order_id']);

        //Get Product
        $product = $this->_productRepository->getById($record['product_id'], false, $record['store_id']);

        //Get Product Thumbnail
        $thumbnail = ($product->getThumbnail() === 'no_selection') ? '' : $store->getBaseUrl(UrlInterface::URL_TYPE_MEDIA) . 'catalog/product' . $product->getThumbnail();

        //Get Attribute Set Name
        $attributeSetId = $product->getAttributeSetId();
        $attributeSetName = '';
        if (!empty($attributeSetId)) {
            $attributeSet = $this->_attributeSet->get($attributeSetId);
            $attributeSetName = $attributeSet->getAttributeSetName();
        }

        // Order link
        $orderLink = $store->getUrl('sales/order/view', ['order_id' => $record['order_id']]);

        // Invoice download link
        $invoiceDlLink = $store->getUrl('sales/order/printInvoice', ['order_id' => $record['order_id']]);

        $result['key'] = strtolower($order->getCustomerEmail());
        $result['order_number'] = $order->getIncrementId();

        // Populate $result with mandatory data
        $result['product_uri'] = $product->getProductUrl();

        //$result['ProductOptions'] = $this->getFormattedOptions($this->_config->unserializeConfigData($record['product_options']));
        $result['product_main_thumbnail'] = $thumbnail;
        $result['product_attribute_set'] = $attributeSetName;
        $result['order_link'] = $orderLink;
        $result['invoice_download_link'] = $invoiceDlLink;
        $result['shipping_address'] = $this->getFormattedAddress($order->getShippingAddress());
        $result['billing_address'] = $this->getFormattedAddress($order->getBillingAddress());
        $result['guest_customer_firstname'] = ($order->getCustomerFirstname()) ? $order->getCustomerFirstname() : $order->getBillingAddress()->getFirstName();
        $result['guest_customer_lastname'] = ($order->getCustomerLastname()) ? $order->getCustomerLastname() : $order->getBillingAddress()->getLastName();
        $result['order_status'] = $order->getStatus();

        return $result;
    }

    /**
     * Format Address
     *
     * @param $address
     * @return string
     */
    public function getFormattedAddress($address)
    {
        if ($address) {
            $html = [];
            $street = $address->getStreet();
            foreach ($street as $streetLine) {
                $html[] = $streetLine;
            }
            $html[] = $address->getPostcode();
            $html[] = $address->getCity();
            $html[] = $address->getCountry();

            $add = implode(' - ', $html);
            return rtrim($add, '- ');
        } else {
            return '';
        }
    }

    /**
     * {@inheritDoc}
     */
    public function getObjectData($primary, $id)
    {
        $connection = $this->resourceConnection->getConnection();
        $tableName = $this->resourceConnection->getTableName('sales_order_item'); //gives table name with prefix

        $only_simple = $this->_config->getConfigValue('mappconnector/order_mapping/export_only_simple');

        //Select Data from table
        $sql = "SELECT * FROM {$tableName} WHERE {$primary} = {$id}";

        if ($only_simple == 1) {
            $sql .= ' AND product_type = "simple" ';
        }

        return $connection->fetchAll($sql);
    }

    /**
     * @param $record
     * @return array
     * @throws NoSuchEntityException
     * @throws Exception
     */
    public function getAdditionalData($record)
    {
        $result = [];

        //Prendo le informazioni dai moduli correlati
        $result = array_merge(
            $result,
            $this->getRelatedModelData($this->_order, $record['order_id'])
        );

        //order_item_product
        $map = $this->_config->getNormalizedConfigObjectByPath('mappconnector/order_mapping/order_item_product', 'order_product');
        $product = $this->_productRepository->getById($record['product_id'], false, $record['store_id']);

        $p = [];
        foreach ($map as $k => $m) {
            $v = $product->getResource()->getAttribute($k)->getFrontend()->getValue($product);
            if (is_array($v)) {
                $v = implode(',', $v);
            }
            $p[$m] = (!is_object($v)) ? $v : '';
        }

        $result = array_merge(
            $result,
            $p
        );

        return $result;
    }

    /**
     * @param string $type
     * @param string $last_index
     * @return mixed
     * @throws Exception
     */
    public function getAllData($type = 'full', $last_index = '')
    {
        if ($type == 'full') {
            //Full
            return $this->_order->getModel()->getCollection()->getData();
        } else {
            //Incremental
            $connection = $this->resourceConnection->getConnection();
            $tableName = $this->resourceConnection->getTableName('sales_order');
            $tableNameJobs = $this->resourceConnection->getTableName('filoblu_mappconnector_jobs');

            $order_statuses = explode(',', $this->_config->getConfigValue('mappconnector/order_mapping/order_statuses_to_export'));

            $os = "";
            foreach ($order_statuses as $order_status) {
                $os .= "'" . $order_status . "',";
            }
            $os = rtrim($os, ',');

            $sql = "SELECT
                      ce.entity_id as entity_id
                    FROM
                      " . $tableName . " AS ce
                    LEFT JOIN
                      " . $tableNameJobs . " AS mj
                    ON
                      mj.entity_id = ce.entity_id
                    AND
                      mj.entity = 'order'
                    WHERE
                      mj.status is null
                    AND
                      ce.status IN (" . $os . ")";

            return $connection->fetchAll($sql);
        }
    }

    /**
     * @return string
     */
    public function getNormalizedEntityModelPrimary()
    {
        return $this->_order->getEntityModelPrimary();
    }

    /**
     * Format Product Options
     *
     * @param $options
     * @return mixed
     * @throws Exception
     */
    public function getFormattedOptions($options)
    {
        $result = [];
        if (count($options)) {
            if (isset($options['options'])) {
                $result = array_merge($result, $options['options']);
            }
            if (isset($options['additional_options'])) {
                $result = array_merge($result, $options['additional_options']);
            }
            if (!empty($options['attributes_info'])) {
                $result = array_merge($options['attributes_info'], $result);
            }
        }

        $response = '';
        try {
            foreach ($result as $res) {
                $response = implode(', ', $res);
            }
        } catch (Exception $e) {
            $this->_fileLogger->error($e->getMessage(), [], __METHOD__ . ':' . __LINE__);
        }

        return $response;
    }
}
